/*
 * ObjectToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectToStringComposer.h"
#include "../StringApi.hpp"

namespace alib {

void ObjectToStringComposer::compose ( std::ostream & out, const WrapperBaseBase & object ) {
	WrapperBaseDispatcher::dispatch(out, object);
}

void ObjectToStringComposer::compose ( std::ostream & out, const CommonBaseBase & object ) {
	ObjectBaseDispatcher::dispatch(out, object);
}

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const Object& object) {
	compose( out, object.getData ( ) );
}

auto ObjectToStringComposerObject = WrapperBaseDispatcher::RegistratorWrapper<void, Object>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Primitive& primitive) {
	compose(out, primitive.getData());
}

auto ObjectToStringComposerPrimitive = WrapperBaseDispatcher::RegistratorWrapper<void, primitive::Primitive>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& output, const label::Label& label) {
	compose(output, label.getData());
}

auto ObjectToStringComposerLabel = WrapperBaseDispatcher::RegistratorWrapper<void, label::Label>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::Symbol& symbol) {
	compose(out, symbol.getData());
}

auto ObjectToStringComposerSymbol = WrapperBaseDispatcher::RegistratorWrapper<void, alphabet::Symbol>(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const Void&) {
	out << "Void";
}

auto ObjectToStringComposerVoid = ObjectBaseDispatcher::RegistratorWrapper<void, Void>(ObjectToStringComposer::compose);

template < class T, class R >
void ObjectToStringComposer::compose(std::ostream& out, const container::ObjectsPair < T, R > & object) {
	out << '<';
	compose(out, object.first);
	out << ", ";
	compose(out, object.second);
	out << '>';
}

auto ObjectToStringComposerObjectsPair = ObjectBaseDispatcher::RegistratorWrapper<void, container::ObjectsPair < Object, Object > >(ObjectToStringComposer::compose);

template < class T >
void ObjectToStringComposer::compose(std::ostream& out, const container::ObjectsSet < T > & object) {
	out << '[';
	bool first = true;
	for(const T & innerObject : object) {
		if(!first)
			out << ", ";
		else
			first = false;
		compose(out, innerObject);
	}
	out << ']';
}

auto ObjectToStringComposerObjectsSet = ObjectBaseDispatcher::RegistratorWrapper<void, container::ObjectsSet < Object > >(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const primitive::String& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerString = ObjectBaseDispatcher::RegistratorWrapper<void, primitive::String>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Character& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerCharacter = ObjectBaseDispatcher::RegistratorWrapper<void, primitive::Character>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Integer& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerInteger = ObjectBaseDispatcher::RegistratorWrapper<void, primitive::Integer>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Unsigned& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerUnsigned = ObjectBaseDispatcher::RegistratorWrapper<void, primitive::Unsigned>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Bool& primitive) {
	out << (primitive.getData() ? "true" : "false");
}

auto ObjectToStringComposerBool = ObjectBaseDispatcher::RegistratorWrapper<void, primitive::Bool>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::LabeledSymbol& symbol) {
	compose(out, symbol.getLabel());
}

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const label::PrimitiveLabel& label) {
	compose(out, label.getData());
}

auto ObjectToStringComposerPrimitiveLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::PrimitiveLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::HexavigesimalLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerHexavigesimalLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::HexavigesimalLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::ObjectLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerObjectLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::ObjectLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::UniqueLabel& label) {
	out << '<';
	compose(out, label.getLabel().getData());
	out << ", ";
	compose(out, label.getId());
	out << '>';
}

auto ObjectToStringComposerUniqueLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::UniqueLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::FailStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerFailStateLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::FailStateLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::InitialStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerInitialStateLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::InitialStateLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::FinalStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerFinalStateLabel = ObjectBaseDispatcher::RegistratorWrapper<void, label::FinalStateLabel>(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

auto ObjectToStringComposerLabeledSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::LabeledSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BlankSymbol&) {
	out << "#B";
}

auto ObjectToStringComposerBlankSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::BlankSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BottomOfTheStackSymbol&) {
	out << "#T";
}

auto ObjectToStringComposerBottomOfTheStackSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::BottomOfTheStackSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::EndSymbol&) {
	out << "#$";
}

auto ObjectToStringComposerEndSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::EndSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::StartSymbol&) {
	out << "#^";
}

auto ObjectToStringComposerStartSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::StartSymbol>(ObjectToStringComposer::compose);

template < class SymbolType, class RankType >
void ObjectToStringComposer::compose(std::ostream& out, const alphabet::RankedSymbol < SymbolType, RankType > & symbol) {
	compose(out, symbol.getSymbol());
	compose(out, symbol.getRank());
}

auto ObjectToStringComposerRankedSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::RankedSymbol < > >(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BarSymbol&) {
	out << "#|";
}

auto ObjectToStringComposerBarSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::BarSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::VariablesBarSymbol&) {
	out << "#/";
}

auto ObjectToStringComposerVariablesBarSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::VariablesBarSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::SubtreeWildcardSymbol&) {
	out << "#S";
}

auto ObjectToStringComposerSubtreeWildcardSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::SubtreeWildcardSymbol>(ObjectToStringComposer::compose);

template < class SymbolType >
void ObjectToStringComposer::compose(std::ostream& out, const alphabet::NonlinearVariableSymbol < SymbolType > & symbol ) {
	out << "$";
	compose(out, symbol.getSymbol());
}

auto ObjectToStringComposerNonlinearVariableSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::NonlinearVariableSymbol < > >(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::UniqueSymbol& symbol) {
	out << '<';
	compose(out, symbol.getSymbol().getData());
	out << ", ";
	compose(out, symbol.getId());
	out << '>';
}

auto ObjectToStringComposerUniqueSymbol = ObjectBaseDispatcher::RegistratorWrapper<void, alphabet::UniqueSymbol>(ObjectToStringComposer::compose);

} /* namespace alib */
